﻿
namespace Observer.Ejm1
{
    /// <summary>
    /// Ejemplo de sujeto observable o sujeto concreto. Es una clase que mantiene un valor entero en
    /// el rango 0..maximo, donde maximo se establece en la construccion. La clase
    /// dispone de metodos para modificar el valor y para acceder al estado
    /// (valor, maximo). Extiende la clase observable.
    /// </summary>

    public class Contador : Observable
    {
        // Atributos privados que mantienen el estado del contador
        private int Valor;
        private int Maximo;

        /*
         * El constructor inicializa el estado del objeto: el maximo y elvalor
         * inicial, siempre en el rango 0..maximo. Adicionalmente,inicializa la
         * parte de Observable que tiene un Contador...
         */
        public Contador(int valor, int maximo)
        {
            this.Maximo = maximo;
            this.Valor = EnRango(valor);
        }

        // Este metodo privado asegura que un valor n esta entre 0..maximo
        private int EnRango(int n)
        {
            if (n < 0)
            {
                return 0;
            }
            else if (n > Maximo)
            {
                return Maximo;
            }
            else
            {
                return n;
            }
        }

        /**
         *  Estos dos metodos permiten el acceso al estado del contador
         * @return el valor del contador
         */
        public int GetValor()
        {
            return Valor;
        }

        public int getMaximo()
        {
            return Maximo;
        }

        /*
         * Este metodo afecta al estado: primero se modifica de forma consistenteel
         * estado y despues se notifica a los observadores del cambio
         */
        public void IncrementarContador(int n)
        {
            Valor = EnRango(Valor + n);
            NotificarObservadores();
        }
    }
}
